package cn.estsh.i3plus.ext.mes.apiservice.controller.base.bu;

import cn.estsh.i3plus.ext.mes.api.base.bu.IBuFirstManageService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesFirstOrderDetail;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessFirstItem;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author: xiangming.liao
 * @CreateDate: 2021/01/18 13:33 下午
 * @Description:
 **/
@RestController
@Api(tags = "首检")
@RequestMapping(CommonConstWords.BASE_URL_MES + "/bu-ext/first")
public class BuFirstManageController {
    public static final Logger LOGGER = LoggerFactory.getLogger(BuFirstManageController.class);

    @Autowired
    private IBuFirstManageService firstManageService;

    @PostMapping(value = "/mes-process-first-item/insert")
    @ApiOperation(value = "添加MES_工序首检项目信息")
    public ResultBean insertMesProcessFirstItem(MesProcessFirstItem mesProcessFirstItem) {
        try {

            ValidatorBean.beginValid(mesProcessFirstItem)
                    .notNull(MesExtConstWords.CHECK_STAND, mesProcessFirstItem.getCheckStand());

            mesProcessFirstItem.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            firstManageService.insertMesProcessFirstItem(mesProcessFirstItem);
            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException imppException) {
            LOGGER.error(imppException.getErrorMsg() + "：{}", imppException.getErrorDetail(), imppException);
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-process-first-item/update")
    @ApiOperation(value = "更新MES_工序首检项目")
    public ResultBean updateMesProcessFirstItem(MesProcessFirstItem mesProcessFirstItem) {
        try {

            if (mesProcessFirstItem.getId() == null || mesProcessFirstItem.getId().longValue() < 1) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }

            firstManageService.updateMesProcessFirstItem(mesProcessFirstItem, AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-process-first-item/query")
    @ApiOperation(value = "按条件分页查询MES_工序首检项目信息", notes = "按条件分页查询MES_工序首检项目信息")
    public ResultBean queryMesProcessFirstItemByPager(MesProcessFirstItem mesProcessFirstItem, Pager pager) {
        try {
            mesProcessFirstItem.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success().setListPager(firstManageService.queryMesProcessFirstItemByPager(mesProcessFirstItem, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-process-first-item/update-status-batch")
    @ApiOperation(value = "批量禁用/启用详情", notes = "批量禁用/启用详情")
    public ResultBean updateMesProcessFirstItemStatus(String[] idsStr, int status) {
        try {
            ConvertBean.modelSafeArrayNumber(idsStr, true);
            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            firstManageService.updateMesProcessFirstItemStatus(idsStr, status, user.getUserName());
            return ResultBean.success("修改成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/mes-first-order-detail/report-query")
    @ApiOperation(value = "首检单(更改为Q管理)明细数据查询报表", notes = "首检单（更改为Q管理）明细数据查询报表")
    public ResultBean queryFirstOrderDetailReportByPager(MesFirstOrderDetail mesFirstOrderDetail, Pager pager) {
        try {
            mesFirstOrderDetail.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success().setListPager(firstManageService.queryFirstOrderDetailReportByPager(mesFirstOrderDetail, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/mes-first-order-detail/by-id/update")
    @ApiOperation(value = "点击工单首检查询报表第中的品质确认和班组确认按钮")
    public ResultBean updateMesFirstOrderDetailById(@RequestBody List<MesFirstOrderDetail> mesFirstOrderDetailList) {
        try {
            firstManageService.updateMesFirstOrderDetailById(mesFirstOrderDetailList, AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
