package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IEquipmentParamService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesEquipmentCheck;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesEquipmentParam;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesEquipmentParamRepository;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCenterRepository;
import cn.estsh.i3plus.pojo.mes.sqlpack.MesHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

/**
 * @author yumingxing
 * @version 1.0
 * @date 2021/2/2 15:19
 **/
@Service
public class EquipmentParamService implements IEquipmentParamService {

    @Autowired
    private MesEquipmentParamRepository equipmentParamRepository;
    @Autowired
    private MesWorkCenterRepository workCenterRepository;

    /**
     * 新增
     *
     * @param equipmentParam
     * @return
     */
    @Override
    public MesEquipmentParam insert(MesEquipmentParam equipmentParam) {
        ConvertBean.modelInitialize(equipmentParam, AuthUtil.getSessionUser());
        equipmentParam.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(equipmentParam.getParamNo(), "paramNo", ddlPackBean);
        MesEquipmentParam mesEquipmentCheck = equipmentParamRepository.getByProperty(ddlPackBean);
        if (!ObjectUtils.isEmpty(mesEquipmentCheck)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode(), "数据已存在");
        }

        return equipmentParamRepository.insert(equipmentParam);
    }

    /**
     * 修改
     *
     * @param equipmentParam
     * @return
     */
    @Override
    public MesEquipmentParam update(MesEquipmentParam equipmentParam) {
        //1、判断单据是否存在
        Long id = equipmentParam.getId();
        MesEquipmentParam equipmentParam2 = equipmentParamRepository.getById(id);
        if (ObjectUtils.isEmpty(equipmentParam2)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode(), "自动配置不存在");
        }

        if (!equipmentParam.getEquNo().equals(equipmentParam2.getEquNo())) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(equipmentParam.getEquNo(), "equNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(equipmentParam.getWorkCellCode(), "workCellCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(equipmentParam.getWorkCenterCode(), "workCenterCode", ddlPackBean);
            MesEquipmentParam mesEquipmentCheck = equipmentParamRepository.getByProperty(ddlPackBean);
            if (!ObjectUtils.isEmpty(mesEquipmentCheck)) {
                throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode(), "数据已存在");
            }
        }
        BeanUtils.copyProperties(equipmentParam, equipmentParam2);
        ConvertBean.serviceModelUpdate(equipmentParam2, AuthUtil.getSessionUser().getUserName());
        return equipmentParamRepository.save(equipmentParam2);
    }

    /**
     * 查询
     *
     * @param equipmentParam
     * @param pager
     * @return
     */
    @Override
    public ListPager<MesEquipmentParam> queryByPager(MesEquipmentParam equipmentParam, Pager pager) {
        DdlPackBean packBean = MesHqlPack.getMesAllData(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(equipmentParam.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getNumEqualPack(equipmentParam.getIsValid(), "isValid", packBean);
        DdlPreparedPack.getStringEqualPack(equipmentParam.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(equipmentParam.getEquNo(), "equNo", packBean);
        pager = PagerHelper.getPager(pager, equipmentParamRepository.findByHqlWhereCount(packBean));
        return new ListPager(equipmentParamRepository.findByHqlWherePage(packBean, pager), pager);
    }

    /**
     * 根据
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public int updateIsValidById(long id, Integer status) {
        //1、判断单据是否存在
        MesEquipmentParam equipmentParam = equipmentParamRepository.getById(id);
        if (ObjectUtils.isEmpty(equipmentParam)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode(), "自动配置不存在");
        }

        return equipmentParamRepository.updateByProperties(new String[]{"id"}, new Object[]{id},
                new String[]{MesConstWords.IS_VALID, "modifyUser", "modifyDatetime"},
                new Object[]{status, AuthUtil.getSessionUser().getUserName(), TimeTool.getNowTime(true)});
    }


    /**
     * 异常
     *
     * @param softTypeCode          softTypeCode
     * @param errorDetailString     错误消息字符串
     * @param imppExceptionEnumCode 异常代码
     * @param args                  参数
     */
    private void throwImppException(String softTypeCode, String errorDetailString, String imppExceptionEnumCode, Object... args) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(softTypeCode)
                .setErrorCode(imppExceptionEnumCode)
                .setErrorDetail(errorDetailString, args)
                .build();
    }
}
