package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessCheck;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesProcessCheckRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProcess;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProcessRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author: xiangming.liao
 * @CreateDate: 2021/02/03 15:34
 * @Description: 工序点检项目导入
 **/
@Service
public class ProcessCheckExtExcelService implements IExcelImportExtService {

    @Autowired
    private MesProcessCheckRepository mesProcessCheckRDao;

    @Autowired
    private MesProcessRepository mesProcessRDao;

    @Autowired
    private MesPartRepository mesPartRDao;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 工序点检项目导入
     *
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesProcessCheck(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesProcessCheck(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 工序点检项目导入-读取表格
     *
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesProcessCheck(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesProcessCheck集合
        List<MesProcessCheck> processCheckExtList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的 processCode + "&" + checkInfo + "&" + partNo 唯一校验
        Map<String, String> processCheckByOnlyMap = new HashMap<>();

        List<MesProcessCheck> processCheckList = mesProcessCheckRDao.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesProcessCheck>> processCheckMap = CollectionUtils.isEmpty(processCheckList) ? null :
                processCheckList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getProcessCode() + "&" + o.getCheckInfo() + "&" + o.getPartNo()));


        // 查询物料表
        Map<String, Object> partNoEntityMap;
        try {
            partNoEntityMap = MesCommonUtil.getCodeEntityMap(mesPartRDao, MesExtConstWords.PART_NO, organizeCode, "物料表");
        } catch (Exception e) {
            partNoEntityMap = null;
        }

        // 查询工序表
        Map<String, Object> processCodeEntityMap;
        try {
            processCodeEntityMap = MesCommonUtil.getCodeEntityMap(mesProcessRDao, MesExtConstWords.PROCESS_CODE, organizeCode, "工序表");
        } catch (Exception e) {
            processCodeEntityMap = null;
        }

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //工序代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String processCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                String processName = null;
                //点检信息
                String checkInfo = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    checkInfo = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //物料编号
                String partNo = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    partNo = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验工序编号
                if (StringUtils.isEmpty(processCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据必填;";
                } else if (!processCodeEntityMap.containsKey(processCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                } else {
                    processName = ((MesProcess) processCodeEntityMap.get(processCode)).getProcessName();
                }

                //校验点检信息
                if (StringUtils.isEmpty(checkInfo)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                }

                //校验零件编号
                if (!StringUtils.isEmpty(partNo)) {
                    if (!partNoEntityMap.containsKey(partNo)) {
                        errorNum++;
                        cellNum += "C;";
                        errorInfo += "第C列数据无效;";
                    }

                }

                //校验唯一性 不重复
                if (!processCheckByOnlyMap.containsKey(processCode + "&" + checkInfo + "&" + partNo)) {
                    processCheckByOnlyMap.put(processCode + "&" + checkInfo + "&" + partNo, String.valueOf(rowNum));
                    //根据工序代码、点击信息、零件号查询中是否存在数据
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(processCheckMap) && processCheckMap.containsKey(processCode + "&" + checkInfo + "&" + partNo)) {
                        errorNum++;
                        cellNum += "A+B+C;";
                        errorInfo += "第A+B+C列数据已经存在数据库;";
                    }
                } else {
                    errorNum++;
                    cellNum += "A+B+C;";
                    errorInfo += "第A+B+C列数据跟第" + processCheckByOnlyMap.get(processCode + "&" + checkInfo + "&" + partNo) +
                            "行的第A+B+C列数据重复;";
                }

                //没有错误的时候，MesProcessCheck
                if (errorNum == 0) {
                    MesProcessCheck mesProcessCheck = new MesProcessCheck();
                    mesProcessCheck.setPartNo(partNo);
                    mesProcessCheck.setProcessCode(processCode);
                    mesProcessCheck.setProcessName(processName);
                    mesProcessCheck.setCheckInfo(checkInfo);
                    mesProcessCheck.setOrganizeCode(organizeCode);
                    ConvertBean.serviceModelInitialize(mesProcessCheck, userName);
                    processCheckExtList.add(mesProcessCheck);
                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultExtModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultExtModel.setExcelList((failRowNum > 0) ? null : processCheckExtList);
        return excelImportResultExtModel;
    }

    /**
     * 工序点检项目导入-数据入库
     *
     * @param excelImportResultExtModel
     */
    private void insertExcelMesProcessCheck(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesProcessCheck> processCheckList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(processCheckList)) {
                for (MesProcessCheck processCheck : processCheckList) {
                    mesProcessCheckRDao.insert(processCheck);
                }
            }
        }
    }
}
