package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerType;
import cn.estsh.i3plus.pojo.mes.repository.MesContainerTypeRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 容器类型导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class SxContainerTypeExcelService implements IExcelImportExtService {

    @Autowired
    private MesContainerTypeRepository containerTypeRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 容器类型导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractContainerType(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelContainerType(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 容器类型导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractContainerType(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesContainerType
        List<MesContainerType> containerTypeList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的容器类型代码唯一校验，key：ctCode，value：row
        Map<String, String> ctCodeMap = new HashMap<>();

        //查询容器类型表
        List<MesContainerType> containerTypes = containerTypeRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesContainerType>> containerTypeMap = CollectionUtils.isEmpty(containerTypes) ? null :
                containerTypes.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesContainerType::getCtCode));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //容器类型代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String ctCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //容器类别
                String ctTypeName = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    ctTypeName = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //容器类型名称
                String ctName = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    ctName = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;
                //校验容器类型代码代码不重复
                if (!ctCodeMap.containsKey(ctCode)) {
                    ctCodeMap.put(ctCode, String.valueOf(rowNum));
                    //校验 容器类型代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(containerTypeMap) && containerTypeMap.containsKey(ctCode)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + ctCodeMap.get(ctCode) + "行的第A列数据重复;";
                }

                Integer ctType = null;
                //校验容器类别有效性
                if (StringUtils.isEmpty(ctTypeName)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                }else{
                    ctType = MesExtEnumUtil.CT_TYPE.descOf(ctTypeName);
                    if(ctType == null){
                        errorNum ++;
                        cellNum += "B;";
                        errorInfo += "第B列数据无效;";
                    }
                }

                //校验容器类型名称
                if (StringUtils.isEmpty(ctName)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                //没有错误的时候，封装MesContainerType
                if (errorNum == 0) {
                    MesContainerType containerType;
                    if(isExist){
                        containerType = containerTypeMap.get(ctCode).get(0);
                        ConvertBean.serviceModelUpdate(containerType, userName);
                    }else{
                        containerType = new MesContainerType();
                        ConvertBean.serviceModelInitialize(containerType, userName);
                    }
                    containerType.setCtCode(ctCode);
                    containerType.setCtName(ctName);
                    containerType.setUseLimit(ctType);
                    containerType.setSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());
                    containerType.setOrganizeCode(organizeCode);

                    containerTypeList.add(containerType);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : containerTypeList);
        return excelImportResultModel;
    }

    /**
     * 容器类型导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelContainerType(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesContainerType> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesContainerType containerType : excelList) {
                    if(StringUtils.isEmpty(containerType.getId())){
                        containerTypeRepository.insert(containerType);
                    }else{
                        containerTypeRepository.save(containerType);
                    }
                }
            }
        }
    }
}
