package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.ep;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesHappenStation;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesHappenStationRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkCenterExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCellRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @PROJECT_NAME: i3plus-mes-panasonic-dev
 * @DESCRIPTION:
 * @USER: xinwang.yi
 * @DATE: 2021-12-13 10:04
 */
@Service
public class HappenStationExcelService implements IExcelImportExtService {
    @Autowired
    private MesHappenStationRepository happenStationRepository;

    @Autowired
    private MesWorkCenterExtRepository workCenterExtRepository;

    @Autowired
    private MesWorkCellRepository workCellRepository;

    @Autowired
    private CommonExtService commonService;

    /**
     * 设备信息导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultModel = this.sheetExtractMesEquipment(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesHappenStation(excelImportResultModel);
        return excelImportResultModel;
    }

    /**
     * 设备信息导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesEquipment(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        //MesEquipment集合
        List<MesHappenStation> mesEquipmentList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorModels = new ArrayList<>();

        //用于EXCEL表中的工作中心代码+工位代码唯一校验，key：workCenterCode&workCellCode，value：row
        Map<String, String> existMap = new HashMap<>();

        // 查询产线表
        Map<String, Object> workCenterCodeEntityMap;
        try {
            workCenterCodeEntityMap = MesCommonUtil.getCodeEntityMap(workCenterExtRepository, MesExtConstWords.WORK_CENTER_CODE, organizeCode, "产线表");
        } catch (Exception e) {
            workCenterCodeEntityMap = null;
        }

        // 查询工位表
        List<MesWorkCell> workCellList = workCellRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesWorkCell>> workCellCodeEntityMap = CollectionUtils.isEmpty(workCellList) ? null :
                workCellList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode()));

        List<MesHappenStation> happenStations = happenStationRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()}
                );
        Map<String, List<MesHappenStation>> happenStationEntityMap = CollectionUtils.isEmpty(happenStations) ? null :
                happenStations.stream().filter(happenStation -> happenStation != null).collect(Collectors.groupingBy(happenStation -> happenStation.getWorkCenterCode() + "&" + happenStation.getWorkCellCode()));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //产线代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String workCenterCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();

                //工位代码
                String workCellCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    workCellCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //设备名称
                String happenStation = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    happenStation = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;

                //校验生产线代码
                if (CollectionUtils.isEmpty(workCenterCodeEntityMap) || !workCenterCodeEntityMap.containsKey(workCenterCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                }

                //校验工位代码
                if (StringUtils.isEmpty(workCellCode)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (StringUtils.isEmpty(errorInfo)) {
                        if (CollectionUtils.isEmpty(workCellCodeEntityMap) || !workCellCodeEntityMap.containsKey(workCenterCode + "&" + workCellCode)) {
                            errorNum++;
                            cellNum += "A+B;";
                            errorInfo += "第A+B列数据无效;";
                        }
                    }
                }

                //备注(可备注匹配的机种类型)
                if (StringUtils.isEmpty(happenStation)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                //校验工作中心代码+工位代码+项目内容+点检方法+类型不重复
                String key = new StringJoiner(MesExtConstWords.AND).add(workCenterCode).add(workCellCode).toString();
                if (!existMap.containsKey(key)) {
                    existMap.put(key, String.valueOf(rowNum));
                    //校验 工作中心代码+工位代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(happenStationEntityMap) && happenStationEntityMap.containsKey(key)) {
                        errorNum ++;
                        cellNum += "A+B;";
                        errorInfo += "第A+B列数据已经存在于数据库;";
                    }
                } else {
                    errorNum ++;
                    cellNum += "A+B;";
                    errorInfo += "第A+B列数据跟第" + existMap.get(key) + "行的第A+B列数据重复;";
                }


                //没有错误的时候，封装MesEquipment
                if (errorNum == 0) {
                    MesHappenStation happenStationNew = new MesHappenStation();
                    happenStationNew.setWorkCenterCode(workCenterCode);
                    happenStationNew.setWorkCellCode(workCellCode);
                    happenStationNew.setHappenStation(happenStation);
                    happenStationNew.setSystemSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());
                    mesEquipmentList.add(happenStationNew);
                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorModel
                    excelImportErrorModels = commonService.getExcelImportErrorModels(excelImportErrorModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : mesEquipmentList);
        return excelImportResultModel;
    }

    /**
     * 发生岗位导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesHappenStation(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesHappenStation> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesHappenStation happenStation : excelList) {
                    happenStationRepository.insert(happenStation);
                }
            }
        }
    }
}
